<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 8477514a (2023-07-28 18:58:18)
// 

use ArfaCMS\Core\ACLObject;
use ArfaCMS\Core\Files;
use ArfaCMS\Core\Profile;
use ArfaCMS\Core\Request;
use ArfaCMS\Core\Settings;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\User;
use ArfaCMS\Core\Util;

defined('_ARFACMS_VALID') or die("Access denied");

$db = $this->initDB();
$pass1 = Request::getSafe('adminPass','');
$pass2 = Request::getSafe('adminPass2','');
$siteDomain= Request::getSafe('siteDomain','');
$sitePort = Request::getInt('sitePort',80);
$siteSSLPort = Request::getInt('siteSSLPort',443);
$adminEmail = Request::getSafe('adminEmail','');
if ($pass1 != $pass2) {
	$err_txt=Text::_('Administrator passwords are not equal');
} else if (mb_strlen($pass1,DEF_CP) < 6) {
	$err_txt=Text::_('Administrator password must be at least 6-chars length');
} else if ($siteDomain== '') {
	$err_txt=Text::_('Enter site domain');
} else if ($adminEmail == '') {
	$err_txt=Text::_('Enter administrator e-mail');
} else $err_txt="";
if ($err_txt) { ?>
<h4 class="invalid">
	<?php echo $err_txt; ?>
</h4>
<form name="frm_install" id="inst_frm" action="install.php"
	method="post">
	<input type="hidden" name="step" value="3" /> 
	<input type="hidden" name="dbHost" value="<?php echo \DatabaseConfig::$dbHost; ?>" /> 
	<input type="hidden" name="dbPort" value="<?php echo \DatabaseConfig::$dbPort; ?>" /> 
	<input type="hidden" name="dbUser" value="<?php echo \DatabaseConfig::$dbUser; ?>" /> 
	<input type="hidden" name="dbPassword" value="<?php echo \DatabaseConfig::$dbPassword; ?>" />
	<input type="hidden" name="dbName" value="<?php echo \DatabaseConfig::$dbName; ?>" /> 
	<input type="hidden" name="dbPrefix" value="<?php echo \DatabaseConfig::$dbPrefix; ?>" />
	<div class="buttons">
		<input type="submit" value="<?php echo Text::_('Go back and fix'); ?>" />
	</div>
</form>
<?php } else {
	\DatabaseConfig::$dbSecret = strtoupper(Util::generateRandomString(10, "", 1));
	$error_detected=false; $err_array=array();
	$dots=str_repeat("&nbsp;&nbsp;.", 30)."&nbsp;";
	$dots="&nbsp;";
	echo "<h3 class=\"title\">".Text::_('Finishing setup')." : </h3>";
	// Write config
	echo "<p class=\"processing\">";
	echo Text::_("Updating config files").$dots;
	if (!$this->writeConfig()) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span></p>";	else echo "<span class=\"ok\">OK</span></p>";
	// SQL files
	echo "<p class=\"processing\">";
	echo Text::_("Creating tables structure").$dots;
	$sqlPath = PATH_INSTALL.'sql'.DS;
	// System structure
	$sqlFile = $sqlPath.'system.sql';
	if (!$this->populateDatabase($db,$sqlFile,$err_array)) {
		$error_detected=true;
	}
	if ($error_detected) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>";	else echo "<span class=\"ok\">OK</span>";
	echo "</p>";
	// System data
	$sqlFilesData = Files::getFiles($sqlPath."data".DS,array(),false);
	if (!$error_detected && $sqlFilesData) {
		echo "<p class=\"processing\">";
		echo Text::_("Inserting system data").$dots;
		foreach ($sqlFilesData as $sqlFile) {
			if (!$this->populateDatabase($db,$sqlPath."data".DS.$sqlFile["filename"],$err_array)) {
				$error_detected=true; break;
			}
		}
		if ($error_detected) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>";	else echo "<span class=\"ok\">OK</span>";
		echo "</p>";
	}
	// Demo data
	$installDemo = Request::getInt('installDemo',0);
	if (!$error_detected && $installDemo) {
		echo "<p class=\"processing\">".Text::_("Inserting DEMO data").$dots;
		$sqlFilesDemo = Files::getFiles($sqlPath."demo".DS,array(),false);
		if ($sqlFilesDemo && count($sqlFilesDemo)) {
			foreach ($sqlFilesDemo as $sqlFile) {
				if (!$this->populateDatabase($db,$sqlPath."demo".DS.$sqlFile["filename"],$err_array)) {
					$error_detected=true; break;
				}
			}
		}
		if (!$error_detected){
			if(!Files::copyFolder(PATH_INSTALL.'data'.DS, ARFACMS_STORAGE_PATH,true)) $error_detected=true;
		}
		if ($error_detected) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>";	else echo "<span class=\"ok\">OK</span>";
		echo "</p>";
	}
	if (!$error_detected && !$installDemo) {
		echo "<p class=\"processing\">";
		echo Text::_("Cleaning data").$dots;
		$sqlFile = $sqlPath.'cleanup.sql';
		if (!$this->populateDatabase($db,$sqlFile, $err_array)) {
			$error_detected=true;
		}
		if ($error_detected) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>";	else echo "<span class=\"ok\">OK</span>";
		echo "</p>";
	}
	// Write config to database
	echo "<p class=\"processing\">";
	echo Text::_("Updating config in database").$dots;
	if (!$this->writeConfigToDatabase($db)) echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>"; else echo "<span class=\"ok\">OK</span>";
	echo "</p>";

	// Setting restructure version
	Settings::setVar("restruct_version", $this->getVersionRevision());

	if (count($err_array)){
		echo "<h4 class=\"invalid\">".Text::_("Detected errors")." : </h4>";
		foreach($err_array as $curent_error){
			echo "<div class=\"found_errors\">".$curent_error."</div>";
		}
	}
	if (!$error_detected) {
		ACLObject::initialize();
		echo "<p class=\"processing\">".Text::_("Creating administrator").$dots;
		if (!User::addUser('admin', 'admin', $pass1, $adminEmail, 1,"",1,0,1)) {
			$error_detected=true;
			echo "<span class=\"invalid\">".Text::_("Failed")." !!!</span>";
		}	else {
			$sql="UPDATE #__users SET u_id=14 WHERE u_login='admin'";
			$db->setQuery($sql);
			$db->query();
			$sql="DELETE FROM #__profiles";
			$db->setQuery($sql);
			$db->query();
			Profile::addProfile(14);
			echo "<span class=\"ok\">OK</span>";
		}
		echo "</p>";
		$siteDomain= Request::getSafe('siteDomain','#');
		$sitePort = Request::getInt('sitePort');
		$siteSSLPort = Request::getInt('siteSSLPort',443);
		if (!$error_detected) { ?>
			<h4 class="setup_ok">
				<?php echo Text::_('Arfa-CMS installed successfully'); ?>
			</h4>
<?php 
			if(Request::getInt('deleteInstaller', 0)) { // Temporary disabled feature
				if(!Files::removeFolder(PATH_INSTALL, 1) || !Files::delete(PATH_SITE."install.php")){ 
?>
				<p class="final_message">
					<?php echo Text::_('Error removing INSTALL directory'); ?> ! ! !
				</p>
				<p class="final_message">
					<?php echo Text::_('Remove INSTALL directory and file install.php before going to site'); ?> ! ! !
				</p>
<?php
				}
				ob_flush();
				header("Location: /".ADMIN_FOLDER."/");
			} else { 
?>
				<p class="final_message">
					<?php echo Text::_('Remove INSTALL directory and file install.php before going to site'); ?> ! ! !
				</p>
			<?php } ?>
			<p class="final_message">
				<?php echo Text::_('Check and save settings in admin mode'); ?>
				! ! !
			</p>
			<p class="final_message">
				<?php echo Text::_('Administrator login is'); ?>
				: admin
			</p>
			<div class="buttons">
				<a class="linkButton" href="<?php echo "/".ADMIN_FOLDER."/"; ?>"><?php echo Text::_('Admin panel'); ?></a>
				&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
				<a class="linkButton" href="<?php echo "/"; ?>"><?php echo Text::_('Site'); ?> </a>
				&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
				<a class="linkButton" target="_blank" href="https://arfa-cms.ru/"><?php echo Text::_('Visit Arfa-CMS site'); ?></a>
			</div>
<?php 
		} else { 
?>
			<h4 class="setup_failed">
				<?php echo Text::_('Arfa-CMS installation failed'); ?>
			</h4>
<?php 
		}
	} else {
?>
		<h4 class="setup_failed">
			<?php echo Text::_('Arfa-CMS installation failed'); ?>
		</h4>
<?php
		
	}
}
?>