<?php
//  Arfa-CMS
//  Copyright (c) Radius17
//  Web: https://arfa-cms.ru/
//  Commercial license https://arfa-cms.ru/article/soglashenie-ob-ispolzovanii.html
//  THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//  Revision: 2be6b5af (2023-06-20 17:48:04)
// 

use ArfaCMS\Core\Menus;
use ArfaCMS\Core\Portal;
use ArfaCMS\Core\Text;
use ArfaCMS\Core\Widget;
use ArfaCMS\DB\Database;

defined('_ARFACMS_VALID') or die("Access denied");

class menuWidget extends Widget {
	private $data_id_attr = "data-id";
	private $data_canonical_attr = "data-canonical-id";

	protected function setParamsMask(){
		parent::setParamsMask();
		$this->addParam("Collapse_widget", "select", 0, false, array(1=>Text::_("Accordeon"), 2=>Text::_("Full screen")), Text::_("Mobile devices"));
		$this->addParam("Collapsed_widget_title", "string", "", false, null, Text::_("Mobile devices"));
		$this->addParam("Hide_main_title_on_mobiles", "boolean", 1);
		$this->addParam("Render_type", "select", 3, true, array("1"=>Text::_("Javascript tree")." (".Text::_("Deprecated").")", "2"=>Text::_("Dropdown"), "3"=>Text::_("Static menu"), "4"=>Text::_("Accordeon")));
		$this->addParam("Hover_dropdown", "boolean", 0, false, null, Text::_("Variants").":<br />".Text::_("Dropdown")." (".Text::_("Except mobiles").")");
		$this->addParam("Expand_active", "boolean", 0, false, null, Text::_("Variants").":<br />".Text::_("Dropdown")."(".Text::_("Except mobiles").")");
		$this->addParam("Menu_ul_id", "string", "");
		$this->addParam("Menu_class", "string", "", false, null, Text::_("Variants").":<br />1) ".Text::_("Empty")."<br />2) navbar-nav<br />3) nav-pills nav-stacked<br />4) nav nav-pills nav-stacked");
		$this->addParam("Menu_li_class", "string", "");
		$this->addParam("Menu_a_class", "string", "");
		$this->addParam("Menu_ul_class", "string", "");
		$this->addParam("Menu_data_toggle_separate", "boolean", 0, false, null, Text::_("Variants").":<br />".Text::_("Dropdown")."<br />".Text::_("Accordeon"));
		$this->addParam("Menu_root_id", "select", "", false, $this->getTopItems());
		$this->addParam("Menu_max_levels", "integer", 0);
		$this->addParam("Hide_menu_titles", "boolean", 0);
		$this->addParam("Translate", "boolean", 0);
	}
	public function prepare() {
		$widget_id = $this->getParam('Widget_ID');
		$ul_id = $this->getParam('Menu_ul_id', false, ($widget_id ? $widget_id."_mnu" : ""));
		$li = $ul_id."_item_".$_SESSION['active_menu_id'];
		$li_canonical = "li[".$this->data_canonical_attr."='".$_SESSION['active_menu_id']."']";
        $render_type = intval($this->getParam('Render_type'));
        $toggle_separate = intval($this->getParam('Menu_data_toggle_separate'));

		if($ul_id) {
            $script = '$(document).ready(function(){';
			switch($render_type){
                case "1":
                    Portal::getInstance()->addScript(LINK_EXTRA."/jquery.plugins/jquery.treeview.min.js");
                    $persist = "location";
                    Portal::getInstance()->AddScriptDeclaration('
						$(document).ready(function(){
							$("#'.$ul_id.'").addClass("tree_menu");
							$("#'.$ul_id.'").treeview({  animated: "fast", collapsed: true, unique: true, persist: "'.$persist.'" });
						});
					');
                    break;
				case "2":
                case "4":
					if($toggle_separate){
						$script.= '    $("#'.$ul_id.' .mnu-toggler-caret-active").click(function(){';
						if($this->getParam('Hover_dropdown') && $render_type == 2){
							$script.= '    if (window.matchMedia("(min-width: 768px)").matches){ return true; }';
						}
						$script.= '        $(this).parents("a").siblings(".mnu-toggler").trigger("click");';
						$script.= '        return false;';
						$script.= '    });';
					}
                    if($this->getParam('Expand_active') && $render_type == 4){
                        $script.= '    $("#'.$ul_id.' .active").addClass("open");';
                    }
                    // Убираем события вызова clearMenus на click по ссылкам меню
                    $script.= " $(document).on('click.bs.dropdown.data-api', '#".$ul_id." a', function (e) { e.stopPropagation(); });";
					break;
                case "3":
                default:
                    break;
			}
			switch($this->getParam('Collapse_widget')){
				case "1":
					break;
				case "2":
                    $script.= "
                        $('#".$widget_id." .navbar-header button').bind('click', function(){
                            $('body').addClass('overlaid');
                            $('#".$widget_id."_wrapper').removeClass('navbar-popup-fullscreen-off').addClass('navbar-popup-fullscreen-on');
                        });
                        $('#".$widget_id." .navbar-popup-fullscreen button').bind('click', function(){
                            $('body').removeClass('overlaid');
                            $('#".$widget_id."_wrapper').removeClass('navbar-popup-fullscreen-on').addClass('navbar-popup-fullscreen-off');
                        });
                    ";
					break;
				default:
					break;
			}
			// Here we should consider mi_canonical_id (data-canonical-id)
			$script.= '    $("#'.$ul_id.' li").removeClass("active");';
			$script.= '    $("#'.$ul_id.' li#'.$li.'").addClass("active");';
			$script.= '    $("#'.$ul_id.' li#'.$li.'").parents("li").addClass("active has_active_child");';

			$script.= '    $("#'.$ul_id.' '.$li_canonical.'").addClass("active");';
			$script.= '    $("#'.$ul_id.' '.$li_canonical.'").parents("li").addClass("active has_active_child");';

            $script.= '});';
            Portal::getInstance()->AddScriptDeclaration($script);
            if($this->getParam('Hide_main_title_on_mobiles')){
                Portal::getInstance()->addStyle("@media (max-width: 767px){#".$widget_id." .wTitle {display:none !important;}}");
            }
		}
	}
	public function render() {
		$widgetHTML="";
		$widget_id = $this->getParam('Widget_ID', false, "widget_".$this->getParam("aw_id"));
		$ul_id = $this->getParam('Menu_ul_id', false, ($widget_id ? $widget_id."_mnu" : ""));
		$ul_hide_menu_titles = $this->getParam('Hide_menu_titles');
		$ul_class = $this->getParam('Menu_class').($ul_hide_menu_titles ? " hide_titles" : "");
		$ul_li_class = $this->getParam('Menu_li_class');
		$ul_ul_class = $this->getParam('Menu_ul_class');
		$ul_a_class = $this->getParam('Menu_a_class');
		$data_toggle = "";
		$data_toggle_separate = $this->getParam('Menu_data_toggle_separate');
		$root_id = $this->getParam('Menu_root_id');
		$translate = $this->getParam('Translate');
		$max_levels= $this->getParam('Menu_max_levels');
		switch($this->getParam('Render_type')){
			case "1":
				break;
			case "2":
				$ul_class = "nav".($ul_class ? " " : "").$ul_class.($this->getParam('Hover_dropdown') ? " hover-dropdown" : "");
				$ul_li_class = "dropdown".($ul_li_class ? " " : "").$ul_li_class;
				$ul_a_class = "dropdown-toggle".($ul_a_class ? " " : "").$ul_a_class;
				$ul_ul_class = "dropdown-menu".($ul_ul_class ? " " : "").$ul_ul_class;
				$data_toggle = "dropdown".($data_toggle ? " " : "").$data_toggle;
				break;
			case "3":
				break;
			case "4":
				$ul_class = "nav".($ul_class ? " " : "").$ul_class;
				$ul_li_class = "dropdown".($ul_li_class ? " " : "").$ul_li_class;
				$ul_a_class = "dropdown-toggle".($ul_a_class ? " " : "").$ul_a_class;
				$ul_ul_class = "dropdown-menu".($ul_ul_class ? " " : "").$ul_ul_class;
				$data_toggle = "dropdown".($data_toggle ? " " : "").$data_toggle;
				/*
				$ul_class.= ($ul_class ? " " : "")."nav";
				$ul_li_class.= ($ul_li_class ? " " : "")."dropdown";
				$ul_a_class.= ($ul_a_class ? " " : "")."dropdown-toggle";
				$ul_ul_class.= ($ul_ul_class ? " " : "")."dropdown-menu";
				$data_toggle.= ($data_toggle ? " " : "")."dropdown";
				*/
				break;
			default:

				break;

		}
		if ($root_id) {
			$model = new Menus(false, true, $ul_class, $ul_li_class, $ul_a_class, $ul_ul_class, $data_toggle, $data_toggle_separate);
			$menuHTML = $model->render($root_id, $ul_id, $translate, $max_levels);
            $auto_close="true";
            if($this->getParam('Render_type') == 4) $auto_close="false";
			$menuHTML = "<div class=\"menu-tree-wrapper tree-wrapper menu-render-type-".$this->getParam('Render_type')."\" data-bs-auto-close=\"".$auto_close."\">".$menuHTML."</div>";
			switch($this->getParam('Collapse_widget')){
				case "1":
					$adaptive_menu_title = $this->getParam('Collapsed_widget_title');
					$widgetHTML.= "<div class=\"navbar-header\"><span class=\"topmenu_label visible-xs\">".$adaptive_menu_title."</span>";
					$widgetHTML.= "<button type=\"button\" class=\"btn btn-navbar navbar-toggle\" data-toggle=\"collapse\" data-target=\"#".$widget_id."_wrapper\"><i class=\"glyphicon glyphicon-menu-hamburger\"></i></button>";
					$widgetHTML.= "</div>";
					$widgetHTML.= "<div id=\"".$widget_id."_wrapper\" class=\"collapse navbar-collapse\">".$menuHTML."</div>";
					break;
				case "2":
					$adaptive_menu_title = $this->getParam('Collapsed_widget_title');
					$widgetHTML.= "<div class=\"navbar-header\"><span class=\"topmenu_label visible-xs\">".$adaptive_menu_title."</span>";
					$widgetHTML.= "<button type=\"button\" class=\"btn btn-navbar navbar-toggle\" data-toggle=\"popup\" data-target=\"#".$widget_id."_wrapper\"><i class=\"glyphicon glyphicon-menu-hamburger\"></i></button>";
					$widgetHTML.= "</div>";
					$widgetHTML.= "<div id=\"".$widget_id."_wrapper\" class=\"navbar-popup-fullscreen navbar-popup-fullscreen-off\">";
					//$widgetHTML.= "<div class=\"float-fix\"><button type=\"button\" class=\"btn close\" data-toggle=\"popup\" data-target=\"#".$widget_id."_wrapper\"><i class=\"glyphicon glyphicon-remove\"></i></button></div>";
					$widgetHTML.= "<button type=\"button\" class=\"btn close\" data-toggle=\"popup\" data-target=\"#".$widget_id."_wrapper\"><i class=\"glyphicon glyphicon-remove\"></i></button>";
					$widgetHTML.= $menuHTML;
					$widgetHTML.= "</div>";
					break;
				default:
					$widgetHTML = $menuHTML;
					break;
			}

		}
		return $widgetHTML;
	}
	public function getTopItems(){
		if(!defined('_ADMIN_MODE')) return null;
		$sql="SELECT mi_id AS id, mi_name AS name FROM #__menus WHERE mi_parent_id=0 ORDER by mi_ordering";
		Database::getInstance()->setQuery($sql);
		return Database::getInstance()->loadObjectList();
	}
}

?>